{ File   : SBE2TXT.dpr
  Author : Deniz Oezmen
  Created: 2007-01-20
  Changed: 2007-01-20

  The Last Express SBE converter.
}
program SBE2TXT;

{$APPTYPE CONSOLE}

uses
  SysUtils, StrUtils;

type
  TSBEHeader  =  packed record
                  NumLines: Word;
                end;

  TLineHeader = packed record
                  Start,          // really? if yes, what unit?
                  Stop,
                  Line1Len,
                  Line2Len: Word;
                end;

var
  SBEFile: file;               

function Prefix(FileName: string): string;
var
  i: Longint;
begin
  FileName := ExpandFileName(FileName);

  i := Length(FileName) + 1;
  repeat
    Dec(i)
  until (i = 0) or (PosEx('.', FileName, i) > 0);

  if (i = 0) or (PosEx('\', FileName, i + 1) > 0) then
    Result := FileName
  else
    Result := Copy(FileName, 1, i - 1)
end;

function OpenSBEFile(var SBEFile: file; SBEName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(SBEFile, SBEName);

  {$I-}
  Reset(SBEFile, 1);
  {$I+}
  Result := (IOResult = 0)
end;

function ReadHeader(var SBEFile: file; out SBEHeader: TSBEHeader): Boolean;
var
  BytesRead: Longint;
begin
  BlockRead(SBEFile, SBEHeader, SizeOf(SBEHeader), BytesRead);

  Result := (BytesRead = SizeOf(SBEHeader))
end;

procedure ConvertSBE(SBEName: string; const SBEHeader: TSBEHeader);
const
  BOM : Word       = $feff;
  CRLF: WideString = #013#010;
var
  i         : Longint;
  Line,
  Line1,
  Line2     : WideString;
  OutFile   : file;
  LineHeader: TLineHeader;
begin
  SBEName := Prefix(SBEName) + '.txt';

  // open output and write UTF BOM marker as well as header
  FileMode := fmOpenWrite;
  AssignFile(OutFile, SBEName);
  {$I-}
  Rewrite(OutFile, 1);
  BlockWrite(OutFile, BOM, 2);
  Line := 'NumLines = ' + IntToStr(SBEHeader.NumLines) + CRLF;
  BlockWrite(OutFile, Line[1], Length(Line) * 2);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn('  Write error.');
    Exit
  end;

  // read  and convert lines
  for i := 1 to SBEHeader.NumLines do
  begin
    {$I-}
    BlockRead(SBEFile, LineHeader, SizeOf(LineHeader));
    {$I+}
    if IOResult <> 0 then
    begin
      if Eof(SBEFile) then
        WriteLn('  Premature end of file.')
      else
        WriteLn('  Read error.');
      Break
    end;

    SetLength(Line1, LineHeader.Line1Len);
    SetLength(Line2, LineHeader.Line2Len);

    {$I-}
    BlockRead(SBEFile, Line1[1], Length(Line1) * 2);
    BlockRead(SBEFile, Line2[1], Length(Line2) * 2);
    {$I+}
    if IOResult <> 0 then
    begin
      if Eof(SBEFile) then
        WriteLn('  Premature end of file.')
      else
        WriteLn('  Read error.');
      Break
    end;

    {$I-}
    Line := '{' + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);

    Line := '  Start = ' + IntToStr(LineHeader.Start) + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);

    Line := '  Stop = ' + IntToStr(LineHeader.Stop) + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);

    Line := '  Line1 = "' + Line1 + '"' + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);

    Line := '  Line2 = "' + Line2 + '"' + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);

    Line := '}' + CRLF;
    BlockWrite(OutFile, Line[1], Length(Line) * 2);
    {$I+}
    if IOResult <> 0 then
    begin
      WriteLn('  Write error.');
      Break
    end
  end;

  Finalize(Line);
  Finalize(Line1);
  Finalize(Line2);

  CloseFile(OutFile)
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  SBEName  : string;
  SearchRec: TSearchRec;
  PFilePara: PChar;
  SBEHeader: TSBEHeader;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  // search for all files matching FilePara
  if FindFirst(PFilePara, faAnyFile - faDirectory,
    SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      SBEName := Dir + FileName;

      WriteLn(SBEName);

      // if this file is accessible, analyze it
      if OpenSBEFile(SBEFile, SBEName) then
      begin
        if ReadHeader(SBEFile, SBEHeader) then
          ConvertSBE(SBEName, SBEHeader)
        else
          WriteLn('  Not a valid SBE file.');

        Close(SBEFile)
      end
      else
        WriteLn('  Not accessible.')
    until FindNext(SearchRec) <> 0
  end
  else
    WriteLn(FilePara, ' not found.')
end;

begin
  if ParamCount >= 1 then
    ParseFiles(ParamStr(1))
  else
    WriteLn('Incorrect number of parameters.')
end.
